-- Copyright 2023 by Todd Hundersmarck (ThundR)
-- All Rights Reserved
THMain = {} local THMain_mt = Class(THMain, THCore) function THMain:new(dataKey, mt) mt = Utils.getNoNil(mt, THMain_mt) local newEnv = THCore:new(dataKey, mt) if newEnv then newEnv.itemConfig = {} newEnv.storeSettings = { safeMode = true, hideItemsFromStore = false, disableItemLeasing = false, itemCountIsGlobal = false } end return newEnv end function THMain.initializeXMLSchema() local xmlSchema = THMain.xmlSchema local storeItemsKey = "thStoreExtension.storeItems" xmlSchema:register(XMLValueType.BOOL, storeItemsKey.."#safeMode", "Enables/Disables store item safe mode", true) xmlSchema:register(XMLValueType.BOOL, storeItemsKey.."#hideFromStore", "Hide all store items by default", false) xmlSchema:register(XMLValueType.BOOL, storeItemsKey.."#disableLeasing", "Globaly disables item leasing", false) xmlSchema:register(XMLValueType.BOOL, storeItemsKey.."#countIsGlobal", "Ignore farmId for maxCount", false) xmlSchema:register(XMLValueType.STRING, storeItemsKey..".storeItem(?)#xmlFilename", "Store item xml filename") xmlSchema:register(XMLValueType.INT, storeItemsKey..".storeItem(?)#maxCount", "Number of items available") xmlSchema:register(XMLValueType.BOOL, storeItemsKey..".storeItem(?)#showInStore", "Show item in store") xmlSchema:register(XMLValueType.BOOL, storeItemsKey..".storeItem(?)#allowLeasing", "Allow leasing of store item") xmlSchema:register(XMLValueType.BOOL, storeItemsKey..".storeItem(?)#countIsGlobal", "Ignore farmId for maxCount", false) end function THMain:initStoreItem(storeItem) if self:argIsValid(type(storeItem) == "table", "storeItem", storeItem) then local itemData = self:getDataTable(storeItem) if not itemData then itemData = self:createDataTable(storeItem) local storeSettings = self.storeSettings local safeMode = storeSettings.safeMode local hideItemsFromStore = storeSettings.hideItemsFromStore local disableItemLeasing = storeSettings.disableItemLeasing local itemCountIsGlobal = storeSettings.itemCountIsGlobal itemData.xmlFileId = storeItem.xmlFilenameLower itemData.defaultShowInStore = storeItem.showInStore itemData.defaultMaxItemCount = storeItem.maxItemCount itemData.defaultAllowLeasing = storeItem.allowLeasing itemData.countIsGlobal = itemCountIsGlobal if hideItemsFromStore and storeItem.showInStore == true then storeItem.showInStore = false end if disableItemLeasing and storeItem.allowLeasing == true then storeItem.allowLeasing = false end local itemConfig = self:getStoreItemConfig(itemData.xmlFileId) if itemConfig then itemConfig.info = storeItem storeItem.showInStore = itemData.defaultShowInStore local cfgMaxCount = itemConfig.maxCount local cfgShowInStore = itemConfig.showInStore local cfgAllowLeasing = itemConfig.allowLeasing local cfgCountIsGlobal = itemConfig.dcountIsGlobal if cfgMaxCount ~= nil then if cfgMaxCount ~= storeItem.maxItemCount then storeItem.maxItemCount = cfgMaxCount end end if cfgShowInStore ~= nil then if cfgShowInStore ~= storeItem.showInStore then if cfgShowInStore == false or not safeMode then storeItem.showInStore = cfgShowInStore else self:warningMsg("Cannot force store visibility for vanilla hidden storeItem %q", itemData.xmlFileId) end end end if cfgAllowLeasing ~= nil then if cfgAllowLeasing ~= storeItem.allowLeasing then if cfgAllowLeasing and not StoreItemUtil.getIsLeasable(storeItem) then self:warningMsg("Cannot allow leasing for non leasable storeItem %q", itemData.xmlFileId) else storeItem.allowLeasing = cfgAllowLeasing end end end if cfgCountIsGlobal ~= nil then itemData.countIsGlobal = cfgCountIsGlobal end itemData.fullXMLFilename = itemConfig.fullXMLFilename itemData.customEnv = itemConfig.customEnv end end return true end return false end function THMain:loadStoreItems(xmlFile, baseDir, customEnv) local storeItemsFilename = nil local success = false customEnv = Utils.getNoNil(customEnv, "") if self:argIsValid(type(customEnv) == "string", "customEnv", customEnv) then local configModFilename, configModBaseDir = self:getConfigModFilename("storeExtension.xml") if configModFilename then baseDir = configModBaseDir customEnv = self.configModName storeItemsFilename = configModFilename else if customEnv == self.mapModName then storeItemsFilename = getXMLString(xmlFile, "map.thStoreExtension#filename") if Utils.getNoNil(storeItemsFilename, "") ~= "" then storeItemsFilename = Utils.getFilename(storeItemsFilename, baseDir) else customEnv = self.modName end else customEnv = self.modName end end if customEnv == self.modName then baseDir = self.modPath storeItemsFilename = baseDir.."xml/storeExtension.xml" end if Utils.getNoNil(storeItemsFilename, "") ~= "" then local storeItemsXMLFile = XMLFile.loadIfExists("THStoreItems", storeItemsFilename, THMain.xmlSchema) if Utils.getNoNil(storeItemsXMLFile, 0) ~= 0 then success = self:loadStoreItemsFromXMLFile(storeItemsXMLFile, baseDir) XMLFile.delete(storeItemsXMLFile) end end end return success end function THMain:loadStoreItemsFromXMLFile(xmlFile, baseDir) if self:argIsValid(type(xmlFile) == "table", "xmlFile", xmlFile) and self:argIsValid(type(baseDir) == "string", "baseDir", baseDir) then local storeItemsKey = "thStoreExtension.storeItems" local safeMode = xmlFile:getValue(storeItemsKey.."#safeMode", true) local hideItemsFromStore = xmlFile:getValue(storeItemsKey.."#hideFromStore", false) local disableItemLeasing = xmlFile:getValue(storeItemsKey.."#disableLeasing", false) local itemCountIsGlobal = xmlFile:getValue(storeItemsKey.."#countIsGlobal", false) local storeSettings = self.storeSettings storeSettings.safeMode = safeMode storeSettings.hideItemsFromStore = hideItemsFromStore storeSettings.disableItemLeasing = disableItemLeasing storeSettings.itemCountIsGlobal = itemCountIsGlobal xmlFile:iterate(storeItemsKey..".storeItem", function(_, pKey) local vXMLFilename = xmlFile:getValue(pKey.."#xmlFilename") local vMaxCount = xmlFile:getValue(pKey.."#maxCount") local vShowInStore = xmlFile:getValue(pKey.."#showInStore") local vAllowLeasing = xmlFile:getValue(pKey.."#allowLeasing") local vCountIsGlobal = xmlFile:getValue(pKey.."#countIsGlobal") if Utils.getNoNil(vXMLFilename, "") ~= "" then local vCustomEnv = nil vXMLFilename, vCustomEnv = self:getFilename(vXMLFilename) if vXMLFilename then local vXMLFileId = vXMLFilename local vFullXMLFilename = vXMLFilename if Utils.getNoNil(vCustomEnv, "") ~= "" then local vModInfo = self:getLoadedMod(vCustomEnv) if vModInfo then local vModPath = vModInfo.modDir vXMLFileId = vModPath..vXMLFilename vFullXMLFilename = vXMLFileId end else vFullXMLFilename = getAppBasePath()..vXMLFilename end vXMLFileId = vXMLFileId:lower() self.itemConfig[vXMLFileId] = { xmlFileId = vXMLFileId, xmlFilename = vXMLFilename, fullXMLFilename = vFullXMLFilename, customEnv = vCustomEnv, maxCount = vMaxCount, showInStore = vShowInStore, allowLeasing = vAllowLeasing, countIsGlobal = vCountIsGlobal } end end end) return true end return false end function THMain:getStoreItem(storeItem, verbose, raise) if not verbose or self:argIsValid(verbose == true, "Verbose", verbose) then local valType = type(storeItem) local itemInfo, itemData = nil,nil if valType == "table" then itemInfo = g_storeManager:getItemByXMLFilename(storeItem.xmlFilenameLower) if itemInfo ~= storeItem then itemInfo = nil end elseif valType == "string" then itemInfo = g_storeManager:getItemByXMLFilename(storeItem:lower()) elseif valType == "number" then itemInfo = g_storeManager:getItemByIndex(storeItem) elseif storeItem ~= nil then verbose = true end if itemInfo then itemData = self:getDataTable(itemInfo) end raise = Utils.getNoNil(raise, verbose) self:msgOnTrue(itemInfo == nil and verbose, raise, THCore.MSG.ARGUMENT_INVALID, "storeItem", storeItem) return itemInfo, itemData end end function THMain:getStoreItemConfig(xmlFileId, verbose, raise) if not verbose or self:argIsValid(verbose == true, "Verbose", verbose) then local valType = type(xmlFileId) local configInfo = nil if valType == "string" then configInfo = self.itemConfig[xmlFileId:lower()] elseif xmlFileId ~= nil then verbose = true end raise = Utils.getNoNil(raise, verbose) self:msgOnTrue(configInfo == nil and verbose, raise, THCore.MSG.ARGUMENT_INVALID, "xmlFileId", xmlFileId) return configInfo end end THStoreManager = {} function THStoreManager:hook_loadMapData(superFunc, xmlFile, missionInfo, baseDir, ...) local function prependFunc() local customEnv = missionInfo.customEnvironment g_thMain:loadStoreItems(xmlFile, baseDir, customEnv) end g_thMain:pcall(prependFunc) return superFunc(self, xmlFile, missionInfo, baseDir, ...) end function THStoreManager:hook_loadItem(superFunc, xmlFilename, baseDir, ...) local loadedItem, ra,rb,rc,rd,re = superFunc(self, xmlFilename, baseDir, ...) local function appendFunc() if type(loadedItem) == "table" and loadedItem.id then local storeItem = self:getItemByIndex(loadedItem.id) if storeItem then g_thMain:initStoreItem(storeItem) end end end g_thMain:pcall(appendFunc) return loadedItem, ra,rb,rc,rd,re end THShopController = {} function THShopController.canBuyItem(self, storeItem) if g_thMain:argIsValid(type(self) == "table" and self:isa(ShopController), "self", self) then local itemInfo, itemData = g_thMain:getStoreItem(storeItem) if itemData then if itemInfo.maxItemCount ~= nil then local playerFarmId = self.playerFarmId if itemData.countIsGlobal then playerFarmId = nil end local totalItems = self.currentMission:getNumOfItems(itemInfo, playerFarmId) if totalItems >= itemInfo.maxItemCount then g_gui:showInfoDialog({ text = self.l10n:getText(ShopController.L10N_SYMBOL.WARNING_TOO_MANY_PLACEABLES) }) return false end end end end return true end function THShopController:hook_canBeBought(superFunc, storeItem, ...) local function prependFunc() if not THShopController.canBuyItem(self, storeItem) then return false end return true end if not g_thMain:call(prependFunc) then return false end return superFunc(self, storeItem, ...) end function THShopController:hook_buyVehicle(superFunc, storeItem, ...) local function prependFunc() if not THShopController.canBuyItem(self, storeItem) then return false end return true end if not g_thMain:call(prependFunc) then return false end return superFunc(self, storeItem, ...) end function THShopController:hook_finalizeBuy(superFunc, ...) local function prependFunc() if not THShopController.canBuyItem(self, self.buyItemFilename) then return false end return true end if not g_thMain:call(prependFunc) then return false end return superFunc(self, ...) end g_thMain = THMain:new("thStoreExtension") if g_thMain then local globalEnv = g_thMain:getGlobalEnv() local modPath = g_thMain.modPath local dataKey = g_thMain.dataKey globalEnv.THStoreExtension = THMain globalEnv.g_thStoreExtension = g_thMain createFolder(g_thMain.settingsPath) g_xmlManager:addCreateSchemaFunction(function() THMain.xmlSchema = XMLSchema.new(dataKey) end) g_xmlManager:addInitSchemaFunction(function() THMain.initializeXMLSchema() end) g_thMain:hookFunction("StoreManager", "loadMapData", THStoreManager) g_thMain:hookFunction("StoreManager", "loadItem", THStoreManager) g_thMain:hookFunction("ShopController", "canBeBought", THShopController) g_thMain:hookFunction("ShopController", "buyVehicle", THShopController) g_thMain:hookFunction("ShopController", "finalizeBuy", THShopController) end